@@ -15,30 +15,46 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """Archive commands for the arc program."""
 import os
+from ..util import PatoolError
 
-def extract_arc (archive, compression, cmd, verbosity, interactive, outdir):
+
+def _add_password_to_options(options, password):
+    """Check password and add it to ARC options."""
+    if password is None:
+        return options
+    if ' ' in password:
+        raise PatoolError("Password for ARC can't contain spaces.")
+    options += 'g%s' % password
+    return options
+
+
+def extract_arc (archive, compression, cmd, verbosity, interactive, outdir, password=None):
     """Extract a ARC archive."""
     # Since extracted files will be placed in the current directory,
     # the cwd argument has to be the output directory.
-    cmdlist = [cmd, 'x', os.path.abspath(archive)]
+    options = _add_password_to_options('x', password)
+    cmdlist = [cmd, options, os.path.abspath(archive)]
     return (cmdlist, {'cwd': outdir})
 
-def list_arc (archive, compression, cmd, verbosity, interactive):
+
+def list_arc (archive, compression, cmd, verbosity, interactive, password=None):
     """List a ARC archive."""
     cmdlist = [cmd]
     if verbosity > 1:
-        cmdlist.append('v')
+        cmdlist.append(_add_password_to_options('v', password))
     else:
-        cmdlist.append('l')
+        cmdlist.append(_add_password_to_options('l', password))
     cmdlist.append(archive)
     return cmdlist
 
-def test_arc (archive, compression, cmd, verbosity, interactive):
-    """Test a ARC archive."""
-    return [cmd, 't', archive]
 
-def create_arc (archive, compression, cmd, verbosity, interactive, filenames):
+def test_arc (archive, compression, cmd, verbosity, interactive, password=None):
+    """Test a ARC archive."""
+    return [cmd, _add_password_to_options('t', password), archive]
+
+
+def create_arc (archive, compression, cmd, verbosity, interactive, filenames, password=None):
     """Create a ARC archive."""
-    cmdlist = [cmd, 'a', archive]
+    cmdlist = [cmd, _add_password_to_options('a', password), archive]
     cmdlist.extend(filenames)
     return cmdlist
