@@ -15,21 +15,30 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """Archive commands for the 7z program."""
 
-def extract_7z(archive, compression, cmd, verbosity, interactive, outdir):
+
+def extract_7z(archive, compression, cmd, verbosity, interactive, outdir, password=None):
     """Extract a 7z archive."""
     cmdlist = [cmd, 'x']
     if not interactive:
         cmdlist.append('-y')
+    if password:
+        cmdlist.append('-p%s' % password)
+    elif not interactive:
+        cmdlist.append('-p-')
     cmdlist.extend(['-o%s' % outdir, '--', archive])
     return cmdlist
 
-def extract_7z_singlefile(archive, compression, cmd, verbosity, interactive, outdir):
-    """Extract a singlefile archive (eg. gzip or bzip2) with '7z e'.
+def extract_7z_singlefile(archive, compression, cmd, verbosity, interactive, outdir, password=None):
+    """Extract a singlefile archive (e.g. gzip or bzip2) with '7z e'.
     This makes sure a single file and no subdirectories are created,
     which would cause errors with patool repack."""
     cmdlist = [cmd, 'e']
     if not interactive:
         cmdlist.append('-y')
+    if password:
+        cmdlist.append('-p%s' % password)
+    elif not interactive:
+        cmdlist.append('-p-')
     cmdlist.extend(['-o%s' % outdir, '--', archive])
     return cmdlist
 
@@ -43,6 +52,7 @@ extract_bzip2 = \
 extract_zip = \
   extract_rar = \
   extract_cab = \
+  extract_chm = \
   extract_arj = \
   extract_cpio = \
   extract_rpm = \
@@ -51,11 +61,15 @@ extract_zip = \
   extract_vhd = \
   extract_7z
 
-def list_7z (archive, compression, cmd, verbosity, interactive):
+def list_7z (archive, compression, cmd, verbosity, interactive, password=None):
     """List a 7z archive."""
     cmdlist = [cmd, 'l']
     if not interactive:
         cmdlist.append('-y')
+    if password:
+        cmdlist.append('-p%s' % password)
+    elif not interactive:
+        cmdlist.append('-p-')
     cmdlist.extend(['--', archive])
     return cmdlist
 
@@ -65,6 +79,7 @@ list_bzip2 = \
   list_compress = \
   list_rar = \
   list_cab = \
+  list_chm = \
   list_arj = \
   list_cpio = \
   list_rpm = \
@@ -76,11 +91,15 @@ list_bzip2 = \
   list_7z
 
 
-def test_7z (archive, compression, cmd, verbosity, interactive):
+def test_7z (archive, compression, cmd, verbosity, interactive, password=None):
     """Test a 7z archive."""
     cmdlist = [cmd, 't']
     if not interactive:
         cmdlist.append('-y')
+    if password:
+        cmdlist.append('-p%s' % password)
+    elif not interactive:
+        cmdlist.append('-p-')
     cmdlist.extend(['--', archive])
     return cmdlist
 
@@ -90,6 +109,7 @@ test_bzip2 = \
   test_compress = \
   test_rar = \
   test_cab = \
+  test_chm = \
   test_arj = \
   test_cpio = \
   test_rpm = \
@@ -101,51 +121,63 @@ test_bzip2 = \
   test_7z
 
 
-def create_7z(archive, compression, cmd, verbosity, interactive, filenames):
+def create_7z(archive, compression, cmd, verbosity, interactive, filenames, password=None):
     """Create a 7z archive."""
     cmdlist = [cmd, 'a']
     if not interactive:
         cmdlist.append('-y')
+    if password:
+        cmdlist.append('-p%s' % password)
+        # encrypt the file list, ie. "patool list" will require a password
+        cmdlist.append('-mhe=on')
     cmdlist.extend(['-t7z', '-mx=9', '--', archive])
     cmdlist.extend(filenames)
     return cmdlist
 
 
-def create_zip(archive, compression, cmd, verbosity, interactive, filenames):
+def create_zip(archive, compression, cmd, verbosity, interactive, filenames, password=None):
     """Create a ZIP archive."""
     cmdlist = [cmd, 'a']
     if not interactive:
         cmdlist.append('-y')
+    if password:
+        cmdlist.append('-p%s' % password)
     cmdlist.extend(['-tzip', '-mx=9', '--', archive])
     cmdlist.extend(filenames)
     return cmdlist
 
 
-def create_xz(archive, compression, cmd, verbosity, interactive, filenames):
+def create_xz(archive, compression, cmd, verbosity, interactive, filenames, password=None):
     """Create an XZ archive."""
     cmdlist = [cmd, 'a']
     if not interactive:
         cmdlist.append('-y')
+    if password:
+        cmdlist.append('-p%s' % password)
     cmdlist.extend(['-txz', '-mx=9', '--', archive])
     cmdlist.extend(filenames)
     return cmdlist
 
 
-def create_gzip(archive, compression, cmd, verbosity, interactive, filenames):
+def create_gzip(archive, compression, cmd, verbosity, interactive, filenames, password=None):
     """Create a GZIP archive."""
     cmdlist = [cmd, 'a']
     if not interactive:
         cmdlist.append('-y')
+    if password:
+        cmdlist.append('-p%s' % password)
     cmdlist.extend(['-tgzip', '-mx=9', '--', archive])
     cmdlist.extend(filenames)
     return cmdlist
 
 
-def create_bzip2(archive, compression, cmd, verbosity, interactive, filenames):
+def create_bzip2(archive, compression, cmd, verbosity, interactive, filenames, password=None):
     """Create a BZIP2 archive."""
     cmdlist = [cmd, 'a']
     if not interactive:
         cmdlist.append('-y')
+    if password:
+        cmdlist.append('-p%s' % password)
     cmdlist.extend(['-tbzip2', '-mx=9', '--', archive])
     cmdlist.extend(filenames)
     return cmdlist
