@@ -14,7 +14,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """Archive commands for the tarfile Python module."""
-from .. import util, py_lzma
+from .. import util
 import tarfile
 
 READ_SIZE_BYTES = 1024*1024
@@ -26,8 +26,8 @@ def list_tar (archive, compression, cmd, verbosity, interactive):
         with tarfile.open(archive) as tfile:
             tfile.list(verbose=verbosity>1)
     except Exception as err:
-        msg = "error listing %s: %s" % (archive, err)
-        raise util.PatoolError(msg)
+        msg = "error listing %s" % (archive, )
+        raise util.PatoolError(msg) from err
     return None
 
 test_tar = list_tar
@@ -36,10 +36,25 @@ def extract_tar (archive, compression, cmd, verbosity, interactive, outdir):
     """Extract a TAR archive with the tarfile Python module."""
     try:
         with tarfile.open(archive) as tfile:
-            tfile.extractall(path=outdir)
+            import os
+
+            def is_within_directory(directory, target):
+                abs_directory = os.path.abspath(directory)
+                abs_target = os.path.abspath(target)
+                prefix = os.path.commonprefix([abs_directory, abs_target])
+                return prefix == abs_directory
+
+            def safe_extract(tar, path=".", members=None, *, numeric_owner=False):
+                for member in tar.getmembers():
+                    member_path = os.path.join(path, member.name)
+                    if not is_within_directory(path, member_path):
+                        raise Exception("Attempted Path Traversal in Tar File")
+                tar.extractall(path, members, numeric_owner=numeric_owner) 
+
+            safe_extract(tfile, path=outdir)
     except Exception as err:
-        msg = "error extracting %s: %s" % (archive, err)
-        raise util.PatoolError(msg)
+        msg = "error extracting %s" % (archive, )
+        raise util.PatoolError(msg) from err
     return None
 
 
@@ -51,8 +66,8 @@ def create_tar (archive, compression, cmd, verbosity, interactive, filenames):
             for filename in filenames:
                 tfile.add(filename)
     except Exception as err:
-        msg = "error creating %s: %s" % (archive, err)
-        raise util.PatoolError(msg)
+        msg = "error creating %s" % (archive, )
+        raise util.PatoolError(msg) from err
     return None
 
 
@@ -62,7 +77,7 @@ def get_tar_mode (compression):
         return 'w:gz'
     if compression == 'bzip2':
         return 'w:bz2'
-    if compression == 'lzma' and py_lzma:
+    if compression == 'lzma':
         return 'w:xz'
     if compression:
         msg = 'pytarfile does not support %s for tar compression'
