@@ -14,25 +14,34 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """Archive commands for the unace program."""
+import os
 
-def extract_ace (archive, compression, cmd, verbosity, interactive, outdir):
+def extract_ace (archive, compression, cmd, verbosity, interactive, outdir, password=None):
     """Extract an ACE archive."""
     cmdlist = [cmd, 'x']
-    if not outdir.endswith('/'):
-        outdir += '/'
+    if not outdir.endswith(os.sep):
+        outdir += os.sep
+    if password:
+        cmdlist.append('-p%s' % password)
     cmdlist.extend([archive, outdir])
     return cmdlist
 
-def list_ace (archive, compression, cmd, verbosity, interactive):
+def list_ace (archive, compression, cmd, verbosity, interactive, password=None):
     """List an ACE archive."""
     cmdlist = [cmd]
     if verbosity > 1:
         cmdlist.append('v')
     else:
         cmdlist.append('l')
+    if password:
+        cmdlist.append('-p%s' % password)
     cmdlist.append(archive)
     return cmdlist
 
-def test_ace (archive, compression, cmd, verbosity, interactive):
+def test_ace (archive, compression, cmd, verbosity, interactive, password=None):
     """Test an ACE archive."""
-    return [cmd, 't', archive]
+    cmdlist = [cmd, 't']
+    if password:
+        cmdlist.append('-p%s' % password)
+    cmdlist.append(archive)
+    return cmdlist
