@@ -15,26 +15,33 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """Archive commands for the unzip program."""
 
-def extract_zip (archive, compression, cmd, verbosity, interactive, outdir):
+def _maybe_add_password(cmdlist, password):
+    if password:
+        cmdlist.extend(['-P', password])
+
+def extract_zip (archive, compression, cmd, verbosity, interactive, outdir, password=None):
     """Extract a ZIP archive."""
     cmdlist = [cmd]
     if verbosity > 1:
         cmdlist.append('-v')
+    _maybe_add_password(cmdlist, password)
     cmdlist.extend(['--', archive, '-d', outdir])
     return cmdlist
 
-def list_zip (archive, compression, cmd, verbosity, interactive):
+def list_zip (archive, compression, cmd, verbosity, interactive, password=None):
     """List a ZIP archive."""
     cmdlist = [cmd, '-l']
     if verbosity > 1:
         cmdlist.append('-v')
+    _maybe_add_password(cmdlist, password)
     cmdlist.extend(['--', archive])
     return cmdlist
 
-def test_zip (archive, compression, cmd, verbosity, interactive):
+def test_zip (archive, compression, cmd, verbosity, interactive, password=None):
     """Test a ZIP archive."""
     cmdlist = [cmd, '-t']
     if verbosity > 1:
         cmdlist.append('-v')
+    _maybe_add_password(cmdlist, password)
     cmdlist.extend(['--', archive])
     return cmdlist
