@@ -14,7 +14,6 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """Utility functions."""
-from __future__ import print_function
 import os
 import sys
 import shutil
@@ -24,61 +23,8 @@ import tempfile
 import time
 import traceback
 import locale
-from . import configuration, ArchiveMimetypes, ArchiveCompressions
-try:
-    from shutil import which
-except ImportError:
-    # from Python 3.3
-    def which(cmd, mode=os.F_OK | os.X_OK, path=None):
-        """Given a command, mode, and a PATH string, return the path which
-        conforms to the given mode on the PATH, or None if there is no such
-        file.
-        `mode` defaults to os.F_OK | os.X_OK. `path` defaults to the result
-        of os.environ.get("PATH"), or can be overridden with a custom search
-        path.
-        """
-        def _access_check(fn, mode):
-            """Check that a given file can be accessed with the correct mode.
-            Additionally check that `fn` is not a directory, as on Windows
-            directories pass the os.access check."""
-            return (os.path.exists(fn) and os.access(fn, mode)
-                    and not os.path.isdir(fn))
-        # If we're given a path with a directory part, look it up directly rather
-        # than referring to PATH directories. This includes checking relative to the
-        # current directory, e.g. ./script
-        if os.path.dirname(cmd):
-            if _access_check(cmd, mode):
-                return cmd
-            return None
-        path = (path or os.environ.get("PATH", os.defpath)).split(os.pathsep)
-        if sys.platform == "win32":
-            # The current directory takes precedence on Windows.
-            if not os.curdir in path:
-                path.insert(0, os.curdir)
-            # PATHEXT is necessary to check on Windows.
-            pathext = os.environ.get("PATHEXT", "").split(os.pathsep)
-            # See if the given file matches any of the expected path extensions.
-            # This will allow us to short circuit when given "python.exe".
-            # If it does match, only test that one, otherwise we have to try
-            # others.
-            if any(cmd.lower().endswith(ext.lower()) for ext in pathext):
-                files = [cmd]
-            else:
-                files = [cmd + ext for ext in pathext]
-        else:
-            # On other platforms you don't have things like PATHEXT to tell you
-            # what file suffixes are executable, so just pass on cmd as-is.
-            files = [cmd]
-        seen = set()
-        for dir in path:
-            normdir = os.path.normcase(dir)
-            if not normdir in seen:
-                seen.add(normdir)
-                for thefile in files:
-                    name = os.path.join(dir, thefile)
-                    if _access_check(name, mode):
-                        return name
-        return None
+from . import configuration, ArchiveMimetypes, ArchiveCompressions, program_supports_compression
+from shutil import which
 
 
 def system_search_path():
@@ -109,10 +55,12 @@ def add_mimedb_data(mimedb):
     mimedb.encodings_map['.lzma'] = 'lzma'
     mimedb.encodings_map['.xz'] = 'xz'
     mimedb.encodings_map['.lz'] = 'lzip'
+    mimedb.encodings_map[".zst"] = "zstd"
     mimedb.suffix_map['.tbz2'] = '.tar.bz2'
     add_mimetype(mimedb, 'application/x-lzop', '.lzo')
     add_mimetype(mimedb, 'application/x-adf', '.adf')
     add_mimetype(mimedb, 'application/x-arj', '.arj')
+    add_mimetype(mimedb, 'application/x-bzip3', '.bz3')
     add_mimetype(mimedb, 'application/x-lzma', '.lzma')
     add_mimetype(mimedb, 'application/x-xz', '.xz')
     add_mimetype(mimedb, 'application/java-archive', '.jar')
@@ -131,6 +79,7 @@ def add_mimedb_data(mimedb):
     add_mimetype(mimedb, 'application/x-lrzip', '.lrz')
     add_mimetype(mimedb, 'application/x-lha', '.lha')
     add_mimetype(mimedb, 'application/x-lzh', '.lzh')
+    add_mimetype(mimedb, 'application/x-lz4', '.lz4')
     add_mimetype(mimedb, 'application/x-rzip', '.rz')
     add_mimetype(mimedb, 'application/x-zoo', '.zoo')
     add_mimetype(mimedb, 'application/x-dms', '.dms')
@@ -147,6 +96,7 @@ def add_mimedb_data(mimedb):
     add_mimetype(mimedb, 'application/zip', '.epub')
     add_mimetype(mimedb, 'application/zip', '.apk')
     add_mimetype(mimedb, 'application/zpaq', '.zpaq')
+    add_mimetype(mimedb, "application/zstd", ".zst")
 
 
 def add_mimetype(mimedb, mimetype, extension):
@@ -195,13 +145,16 @@ def backtick (cmd, encoding='utf-8'):
     return data.decode(encoding)
 
 
-def run (cmd, verbosity=0, **kwargs):
+def run(cmd, verbosity=0, **kwargs):
     """Run command without error checking.
     @return: command return code"""
     # Note that shell_quote_nt() result is not suitable for copy-paste
     # (especially on Unix systems), but it looks nicer than shell_quote().
     if verbosity >= 0:
         log_info("running %s" % " ".join(map(shell_quote_nt, cmd)))
+    if os.name == "nt":
+        # prevent opening of additional consoles when running with pythonw.exe
+        kwargs["creationflags"] = subprocess.CREATE_NO_WINDOW
     if kwargs:
         if verbosity >= 0:
             log_info("    with %s" % ", ".join("%s=%s" % (k, shell_quote(str(v)))\
@@ -250,6 +203,7 @@ Encoding2Mime = {
     'lzma': "application/x-lzma",
     'lzip': "application/x-lzip",
     'xz': "application/x-xz",
+    "zstd": "application/zstd",
 }
 Mime2Encoding = dict([(_val, _key) for _key, _val in Encoding2Mime.items()])
 # libmagic before version 5.14 identified .gz files as application/x-gzip
@@ -291,19 +245,18 @@ def guess_mime_file (filename):
                 encoding = None
     if mime in Mime2Encoding:
         # try to look inside compressed archives
-        cmd = [file_prog, "--brief", "--mime", "--uncompress", filename]
+        cmd = [file_prog, "--brief", "--mime", "--uncompress", "--no-sandbox", filename]
         try:
             outparts = backtick(cmd).strip().split(";")
+            mime2 = outparts[0].split(" ", 1)[0]
         except OSError:
-            # ignore errors, as file(1) is only a fallback
-            return mime, encoding
-        mime2 = outparts[0].split(" ", 1)[0]
+            mime2 = None
         # Some file(1) implementations return an empty or unknown mime type
         # when the uncompressor program is not installed, other
         # implementation return the original file type.
         # The following detects both cases.
         if (mime2 in ('application/x-empty', 'application/octet-stream') or
-            mime2 in Mime2Encoding):
+            mime2 in Mime2Encoding or not mime2):
             # The uncompressor program file(1) uses is not installed
             # or is not able to uncompress.
             # Try to get mime information from the file extension.
@@ -314,7 +267,12 @@ def guess_mime_file (filename):
         elif mime2 in ArchiveMimetypes:
             mime = mime2
             encoding = get_file_mime_encoding(outparts)
-    return mime, encoding
+    # Only return mime and encoding if the given mime can natively support the encoding.
+    if program_supports_compression(ArchiveMimetypes.get(mime), encoding):
+        return mime, encoding
+    else:
+        # If encoding is None, default back to `mime`.
+        return Encoding2Mime.get(encoding, mime), None
 
 
 def guess_mime_file_mime (file_prog, filename):
@@ -350,6 +308,7 @@ FileText2Mime = {
     "Amiga DOS disk": "application/x-adf",
     "ARJ archive data": "application/x-arj",
     "bzip2 compressed data": "application/x-bzip2",
+    "bzip3 compressed data": "application/x-bzip3",
     "cpio archive": "application/x-cpio",
     "ASCII cpio archive": "application/x-cpio",
     "Debian binary package": "application/x-debian-package",
@@ -377,11 +336,10 @@ FileText2Mime = {
     "ZPAQ stream": "application/zpaq",
 }
 
-def guess_mime_file_text (file_prog, filename):
+def guess_mime_file_text(file_prog, filename):
     """Determine MIME type of filename with file(1)."""
-    cmd = [file_prog, "--brief", filename]
     try:
-        output = backtick(cmd).strip()
+        output = run_file_text(file_prog, filename)
     except OSError:
         # ignore errors, as file(1) is only a fallback
         return None
@@ -392,6 +350,12 @@ def guess_mime_file_text (file_prog, filename):
     return None
 
 
+def run_file_text(file_prog, filename):
+    """Get stripped output of file --brief <filename>"""
+    cmd = [file_prog, "--brief", filename]
+    return backtick(cmd).strip()
+
+
 def check_existing_filename (filename, onlyfiles=True):
     """Ensure that given filename is a valid, existing file."""
     if not os.path.exists(filename):
@@ -473,7 +437,7 @@ def tmpfile (dir=None, prefix="temp", suffix=None):
 
 def shell_quote (value):
     """Quote all shell metacharacters in given string value with strong
-    (ie. single) quotes, handling the single quote especially."""
+    (i.e. single) quotes, handling the single quote especially."""
     if os.name == 'nt':
         return shell_quote_nt(value)
     return "'%s'" % value.replace("'", r"'\''")
@@ -489,7 +453,10 @@ def shell_quote_nt (value):
 
 def stripext (filename):
     """Return the basename without extension of given filename."""
-    return os.path.splitext(os.path.basename(filename))[0]
+    basename, _ = os.path.splitext(os.path.basename(filename))
+    if basename.endswith(".tar"):
+        basename, _ = os.path.splitext(basename)
+    return basename
 
 
 def get_single_outfile (directory, archive, extension=""):
@@ -506,20 +473,30 @@ def get_single_outfile (directory, archive, extension=""):
     return outfile + extension
 
 
-def log_error (msg, out=sys.stderr):
+def print_safe(*args, file=sys.stdout):
+    """Print arguments without encoding errors, replacing unknown
+    characters."""
+    msgs = [
+        str(arg).encode(file.encoding, errors="replace").decode()
+        for arg in args
+    ]
+    print(*msgs, file=file)
+
+
+def log_error(msg):
     """Print error message to stderr (or any other given output)."""
-    print("patool error:", msg, file=out)
+    print_safe("patool error:", msg, file=sys.stderr)
 
 
-def log_info (msg, out=sys.stdout):
+def log_info(msg):
     """Print info message to stdout (or any other given output)."""
-    print("patool:", msg, file=out)
+    print_safe("patool:", msg)
 
 
-def log_internal_error(out=sys.stderr, etype=None, evalue=None, tb=None):
+def log_internal_error(etype=None, evalue=None, tb=None):
     """Print internal error message (output defaults to stderr)."""
-    print(os.linesep, file=out)
-    print("""********** Oops, I did it again. *************
+    print_safe(os.linesep, file=sys.stderr)
+    print_safe("""********** Oops, I did it again. *************
 
 You have found an internal error in %(app)s. Please write a bug report
 at %(url)s and include at least the information below:
@@ -527,42 +504,31 @@ at %(url)s and include at least the information below:
 Not disclosing some of the information below due to privacy reasons is ok.
 I will try to help you nonetheless, but you have to give me something
 I can work with ;) .
-""" % dict(app=configuration.AppName, url=configuration.SupportUrl), file=out)
+""" % dict(app=configuration.AppName, url=configuration.SupportUrl), file=sys.stderr)
     if etype is None:
         etype = sys.exc_info()[0]
     if evalue is None:
         evalue = sys.exc_info()[1]
-    print(etype, evalue, file=out)
+    print_safe(etype, evalue, file=sys.stderr)
     if tb is None:
         tb = sys.exc_info()[2]
-    traceback.print_exception(etype, evalue, tb, None, out)
-    print_app_info(out=out)
-    print_locale_info(out=out)
-    print(os.linesep,
-      "******** %s internal error, over and out ********" % configuration.AppName, file=out)
-
-def print_env_info(key, out=sys.stderr):
-    """If given environment key is defined, print it out."""
-    value = os.getenv(key)
-    if value is not None:
-        print(key, "=", repr(value), file=out)
-
-
-def print_locale_info(out=sys.stderr):
-    """Print locale info."""
-    for key in ("LANGUAGE", "LC_ALL", "LC_CTYPE", "LANG"):
-        print_env_info(key, out=out)
-
-
-def print_app_info(out=sys.stderr):
-    """Print system and application info (output defaults to stderr)."""
-    print("System info:", file=out)
-    print(configuration.App, file=out)
-    print("Python %(version)s on %(platform)s" %
-                    {"version": sys.version, "platform": sys.platform}, file=out)
+    traceback.print_exception(etype, evalue, tb, None, sys.stderr)
+    # print system and application info
+    print_safe("System info:", file=sys.stderr)
+    print_safe(configuration.App, file=sys.stderr)
+    print_safe("Python %(version)s on %(platform)s" %
+                    {"version": sys.version, "platform": sys.platform}, file=sys.stderr)
     stime = strtime(time.time())
-    print("Local time:", stime, file=out)
-    print("sys.argv", sys.argv, file=out)
+    print_safe("Local time:", stime, file=sys.stderr)
+    print_safe("sys.argv", sys.argv, file=sys.stderr)
+    # print locale info
+    for key in ("LANGUAGE", "LC_ALL", "LC_CTYPE", "LANG"):
+        value = os.getenv(key)
+        if value is not None:
+            print_safe(key, "=", repr(value), file=sys.stderr)
+    print_safe(os.linesep,
+      "******** %s internal error, over and out ********" %
+      configuration.AppName, file=sys.stderr)
 
 
 def strtime(t):
@@ -587,12 +553,13 @@ def p7zip_supports_rar():
         # Assume RAR support is compiled into the binary.
         return True
     # the subdirectory and codec name
-    codecname = 'p7zip/Codecs/Rar29.so'
+    codecnames = ['p7zip/Codecs/Rar29.so', 'p7zip/Codecs/Rar.so']
     # search canonical user library dirs
     for libdir in ('/usr/lib', '/usr/local/lib', '/usr/lib64', '/usr/local/lib64', '/usr/lib/i386-linux-gnu', '/usr/lib/x86_64-linux-gnu'):
-        fname = os.path.join(libdir, codecname)
-        if os.path.exists(fname):
-            return True
+        for codecname in codecnames:
+            fname = os.path.join(libdir, codecname)
+            if os.path.exists(fname):
+                return True
     return False
 
 
@@ -621,15 +588,19 @@ def append_to_path (path, directory):
     return path + directory
 
 
-def get_nt_7z_dir ():
+def get_nt_7z_dir():
     """Return 7-Zip directory from registry, or an empty string."""
-    # Python 3.x renamed the _winreg module to winreg
+    import winreg
+    import platform
+    python_bits = platform.architecture()[0]
+    keyname = r"SOFTWARE\7-Zip"
     try:
-        import _winreg as winreg
-    except ImportError:
-        import winreg
-    try:
-        key = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, r"SOFTWARE\7-Zip")
+        if python_bits == '32bit' and platform.machine().endswith('64'):
+            # get 64-bit registry key from 32-bit Python
+            key = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, keyname,
+                  0, winreg.KEY_READ | winreg.KEY_WOW64_64KEY)
+        else:
+            key = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, keyname)
         try:
             return winreg.QueryValueEx(key, "Path")[0]
         finally:
@@ -663,7 +634,7 @@ def strlist_with_or (alist):
 
 def is_same_file (filename1, filename2):
     """Check if filename1 and filename2 point to the same file object.
-    There can be false negatives, ie. the result is False, but it is
+    There can be false negatives, i.e. the result is False, but it is
     the same file anyway. Reason is that network filesystems can create
     different paths to the same physical file.
     """
@@ -691,8 +662,9 @@ def link_or_copy(src, dst, verbosity=0):
     except (AttributeError, OSError):
         try:
             shutil.copy(src, dst)
-        except OSError as msg:
-            raise PatoolError(msg)
+        except OSError as err:
+            msg = "error copying %s -> %s" % (src, dst)
+            raise PatoolError(msg) from err
 
 
 def chdir(directory):
